"""
This type stub file was generated by pyright.
"""

from __future__ import annotations

import sys
from argparse import Namespace
from typing import TYPE_CHECKING

"""
``ozi-new`` interactive prompts
"""
if sys.platform != 'win32':
    ...
else:
    ...
if TYPE_CHECKING:
    ...

def interactive_prompt(project: Namespace) -> list[str]: ...
