"""
This type stub file was generated by pyright.
"""

from typing import Any

def main_menu(
    project: Any, output: dict[str, list[str]], prefix: dict[str, str]
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]: ...
def options_menu(
    project: Any, output: dict[str, list[str]], prefix: dict[str, str]
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]: ...
def edit_menu(
    project: Any, output: dict[str, list[str]], prefix: dict[str, str]
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]: ...
